@@ -54,6 +54,7 @@
 #'   default), the first category is used as the reference. If \code{NA}, all
 #'   categories will be predicted, which requires strong priors or carefully
 #'   specified predictor terms in order to lead to an identified model.
+#' @param bhaz Currently for experimental purposes only.
 #' 
 #' @details 
 #'   Below, we list common use cases for the different families.
@@ -189,7 +190,7 @@ brmsfamily <- function(family, link = NULL, link_sigma = "log",
                        link_alpha = "identity", 
                        link_quantile = "logit",
                        threshold = c("flexible", "equidistant"),
-                       refcat = NULL) {
+                       refcat = NULL, bhaz = NULL) {
   slink <- substitute(link)
   .brmsfamily(
     family, link = link, slink = slink,
@@ -202,7 +203,8 @@ brmsfamily <- function(family, link = NULL, link_sigma = "log",
     link_ndt = link_ndt, link_bias = link_bias,
     link_alpha = link_alpha, link_xi = link_xi,
     link_quantile = link_quantile,
-    threshold = threshold, refcat = refcat
+    threshold = threshold, refcat = refcat,
+    bhaz = bhaz
   )
 }
 
@@ -216,7 +218,7 @@ brmsfamily <- function(family, link = NULL, link_sigma = "log",
 # @return an object of 'brmsfamily' which inherits from 'family'
 .brmsfamily <- function(family, link = NULL, slink = link,
                         threshold = c("flexible", "equidistant"),
-                        refcat = NULL, ...) {
+                        refcat = NULL, bhaz = NULL, ...) {
   family <- tolower(as_one_character(family))
   aux_links <- list(...)
   pattern <- c("^normal$", "^zi_", "^hu_")
@@ -283,6 +285,23 @@ brmsfamily <- function(family, link = NULL, link_sigma = "log",
       out$refcat <- as_one_character(refcat, allow_na = TRUE) 
     }
   }
+  if (is_cox(out$family)) {
+    if (!is.null(bhaz)) {
+      if (!is.list(bhaz)) {
+        stop2("'bhaz' should be a list.")
+      }
+      out$bhaz <- bhaz
+    } else {
+      out$bhaz <- list()
+    }
+    # set default arguments
+    if (is.null(out$bhaz$df)) {
+      out$bhaz$df <- 4L
+    }
+    if (is.null(out$bhaz$intercept)) {
+      out$bhaz$intercept <- FALSE
+    }
+  }
   out
 }
 
@@ -423,13 +442,11 @@ family_info.brmsfit <- function(x, y, ...) {
 # provides special handling for certain elements
 combine_family_info <- function(x, y, ...) {
   y <- as_one_character(y)
-  unite <- c("dpars", "type", "specials", "include", "const", "cats")
+  unite <- c("dpars", "type", "specials", "include", "const", "cats", "ad")
   if (y %in% c("family", "link")) {
     x <- unlist(x)
   } else if (y %in% unite) {
     x <- Reduce("union", x)
-  } else if (y == "ad") {
-    x <- Reduce("intersect", x)
   } else if (y == "ybounds") {
     x <- do_call(rbind, x)
     x <- c(max(x[, 1]), min(x[, 2]))
@@ -601,6 +618,26 @@ asym_laplace <- function(link = "identity", link_sigma = "log",
               link_sigma = link_sigma, link_quantile = link_quantile)
 }
 
+# do not export yet!
+# @rdname brmsfamily
+# @export
+zero_inflated_asym_laplace <- function(link = "identity", link_sigma = "log",
+                                       link_quantile = "logit",
+                                       link_zi = "logit") {
+  slink <- substitute(link)
+  .brmsfamily("zero_inflated_asym_laplace", link = link, slink = slink,
+              link_sigma = link_sigma, link_quantile = link_quantile,
+              link_zi = link_zi)
+}
+
+# do not export yet!
+# @rdname brmsfamily
+# @export
+cox <- function(link = "log", bhaz = NULL) {
+  slink <- substitute(link)
+  .brmsfamily("cox", link = link, bhaz = bhaz)
+}
+
 #' @rdname brmsfamily
 #' @export
 hurdle_poisson <- function(link = "log") {
@@ -1378,6 +1415,10 @@ is_dirichlet <- function(family) {
   "dirichlet" %in% family_info(family, "specials")
 }
 
+is_cox <- function(family) {
+  "cox" %in% family_info(family, "specials")
+}
+
 allow_factors <- function(family) {
   specials <- c("binary", "categorical", "ordinal")
   any(specials %in% family_info(family, "specials"))
@@ -1477,10 +1518,8 @@ has_sigma <- function(family) {
 no_sigma <- function(bterms) {
   stopifnot(is.brmsterms(bterms))
   if (is.formula(bterms$adforms$se)) {
-    # call resp_se without evaluating the x argument
-    cl <- rhs(bterms$adforms$se)[[2]]
-    cl[[1]] <- quote(resp_se_no_data)
-    se_only <- isFALSE(attr(eval(cl), "sigma"))
+    se <- eval_rhs(bterms$adforms$se)
+    se_only <- isFALSE(se$flags$sigma)
     if (se_only && use_cov(bterms$autocor)) {
       stop2("Please set argument 'sigma' of function 'se' ",
             "to TRUE when modeling ARMA covariance matrices.")
@@ -1505,7 +1544,8 @@ pred_sigma <- function(bterms) {
 
 # has the model latent residuals to be used in autocor structures
 has_latent_residuals <- function(bterms) {
-  !has_natural_residuals(bterms) && is.cor_arma(bterms$autocor)
+  !has_natural_residuals(bterms) && 
+    (is.cor_arma(bterms$autocor) || is.cor_cosy(bterms$autocor))
 }
 
 # should natural residuals be modeled as correlated?
