@@ -214,6 +214,15 @@ fitted_asym_laplace <- function(draws) {
   )
 }
 
+fitted_zero_inflated_asym_laplace <- function(draws) {
+  fitted_asym_laplace(draws) * (1 - draws$dpars$zi)
+}
+
+fitted_cox <- function(draws) {
+  stop2("Cannot compute expected values of the posterior predictive ",
+        "distribution for family 'cox'.")
+}
+
 fitted_hurdle_poisson <- function(draws) {
   with(draws$dpars, mu / (1 - exp(-mu)) * (1 - hu))
 }
