@@ -41,7 +41,7 @@ stan_response <- function(bterms, data) {
     }
   }
   if (has_ndt(family)) {
-    str_add(out$tdataD) <- glue(
+    str_add(out$tdata_def) <- glue(
       "  real min_Y{resp} = min(Y{resp});\n"
     )
   }
@@ -59,7 +59,7 @@ stan_response <- function(bterms, data) {
     str_add(out$data) <- glue(
       "  vector<lower=0>[N{resp}] se{resp};  // known sampling error\n"
     )
-    str_add(out$tdataD) <- glue(
+    str_add(out$tdata_def) <- glue(
       "  vector<lower=0>[N{resp}] se2{resp} = square(se{resp});\n"
     )
   }
@@ -68,12 +68,23 @@ stan_response <- function(bterms, data) {
       "  int<lower=0,upper=1> dec{resp}[N{resp}];  // decisions\n"
     )
   }
-  has_cens <- has_cens(bterms, data = data)
-  if (has_cens) {
+  if (is.formula(bterms$adforms$rate)) {
+    str_add(out$data) <- glue(
+      "  vector<lower=0>[N{resp}] denom{resp};",
+      "  // response denominator\n"
+    )
+    str_add(out$tdata_def) <- glue(
+      "  // log response denominator\n",
+      "  vector[N{resp}] log_denom{resp} = log(denom{resp});\n"
+    )
+  }
+  if (is.formula(bterms$adforms$cens)) {
+    cens <- eval_rhs(bterms$adforms$cens)
     str_add(out$data) <- glue(
       "  int<lower=-1,upper=2> cens{resp}[N{resp}];  // indicates censoring\n"
     )
-    if (isTRUE(attr(has_cens, "interval"))) {
+    if (cens$vars$y2 != "NA") {
+      # interval censoring is required
       rcens <- str_if(rtype == "int", 
         glue("  int rcens{resp}[N{resp}];"), 
         glue("  vector[N{resp}] rcens{resp};")
@@ -83,7 +94,7 @@ stan_response <- function(bterms, data) {
       )
     }
   }
-  bounds <- get_bounds(bterms, data = data)
+  bounds <- trunc_bounds(bterms, data = data)
   if (any(bounds$lb > -Inf)) {
     str_add(out$data) <- glue(
       "  {rtype} lb{resp}[N{resp}];  // lower truncation bounds;\n"
@@ -95,7 +106,7 @@ stan_response <- function(bterms, data) {
     )
   }
   if (is.formula(bterms$adforms$mi)) {
-    Ybounds <- get_bounds(bterms, data, incl_family = TRUE, stan = TRUE)
+    Ybounds <- trunc_bounds(bterms, data, incl_family = TRUE, stan = TRUE)
     sdy <- get_sdy(bterms, data)
     if (is.null(sdy)) {
       # response is modeled without measurement error
@@ -108,10 +119,11 @@ stan_response <- function(bterms, data) {
         "  int<lower=1> Jmi{resp}[Nmi{resp}];",  
         "  // positions of missings\n"
       )
-      str_add(out$modelD) <- glue(
+      str_add(out$model_def) <- glue(
+        "  // vector combining observed and missing responses\n",
         "  vector[N{resp}] Yl{resp} = Y{resp};\n" 
       )
-      str_add(out$modelC1) <- glue(
+      str_add(out$model_comp_basic) <- glue(
         "  Yl{resp}[Jmi{resp}] = Ymi{resp};\n"
       )
     } else {
@@ -132,6 +144,24 @@ stan_response <- function(bterms, data) {
       )
     }
   }
+  if (is.formula(bterms$adforms$vreal)) {
+    # vectors of real values for use in custom families
+    vreal <- eval_rhs(bterms$adforms$vreal)
+    k <- length(vreal$vars)
+    str_add(out$data) <- cglue(
+      "  // data for custom real vectors\n",
+      "  vector[N{resp}] vreal{seq_len(k)}{resp};\n"
+    )
+  }
+  if (is.formula(bterms$adforms$vint)) {
+    # vectors of integer values for use in custom families
+    vint <- eval_rhs(bterms$adforms$vint)
+    k <- length(vint$vars)
+    str_add(out$data) <- cglue(
+      "  // data for custom integer vectors\n",
+      "  int vint{seq_len(k)}{resp}[N{resp}];\n"
+    )
+  }
   out
 }
 
@@ -156,12 +186,50 @@ stan_autocor <- function(bterms, prior) {
       "  int<lower=0> Kar{p};  // AR order\n",
       "  int<lower=0> Kma{p};  // MA order\n"
     )
-    str_add(out$tdataD) <- glue( 
+    str_add(out$tdata_def) <- glue( 
       "  int max_lag{p} = max(Kar{p}, Kma{p});\n"
     )
-    Kar <- get_ar(autocor)
-    if (Kar) {
-      ar_bound <- subset2(prior, class = "ar", ls = px)$bound
+    ar_bound <- ma_bound <- "<lower=-1,upper=1>"
+    if (!use_cov(autocor)) {
+      err_msg <- "Please set cov = TRUE in ARMA correlation structures"
+      if (!has_natural_residuals) {
+        stop2(err_msg, " for family '", family$family, "'.")
+      }
+      if (is.formula(bterms$adforms$se)) {
+        stop2(err_msg, " when including known standard errors.")
+      }
+      str_add(out$data) <- glue( 
+        "  // number of lags per observation\n",
+        "  int<lower=0> J_lag{p}[N{p}];\n"                
+      )
+      str_add(out$model_def) <- glue(
+        "  // objects storing residuals\n",
+        "  matrix[N{p}, max_lag{p}] Err{p}",
+        " = rep_matrix(0, N{p}, max_lag{p});\n",
+        "  vector[N{p}] err{p};\n"
+      )
+      Y <- str_if(is.formula(bterms$adforms$mi), "Yl", "Y")
+      add_ar <- str_if(get_ar(autocor),
+        glue("    mu{p}[n] += head(Err{p}[n], Kar{p}) * ar{p};\n")             
+      )
+      add_ma <- str_if(get_ma(autocor),
+        glue("    mu{p}[n] += head(Err{p}[n], Kma{p}) * ma{p};\n")             
+      )
+      str_add(out$model_comp_arma) <- glue(
+        "  // include ARMA terms\n",
+        "  for (n in 1:N{p}) {{\n",
+             add_ma,
+        "    err{p}[n] = {Y}{p}[n] - mu{p}[n];\n",
+        "    for (i in 1:J_lag{p}[n]) {{\n",
+        "      Err{p}[n + 1, i] = err{p}[n + 1 - i];\n",
+        "    }}\n",
+             add_ar,
+        "  }}\n"
+      )
+      # in the conditional formulation no boundaries are required
+      ar_bound <- ma_bound <- ""
+    }
+    if (get_ar(autocor)) {
       str_add(out$par) <- glue( 
         "  vector{ar_bound}[Kar{p}] ar{p};  // autoregressive effects\n"
       )
@@ -169,9 +237,7 @@ stan_autocor <- function(bterms, prior) {
         prior, class = "ar", px = px, suffix = p
       )
     }
-    Kma <- get_ma(autocor)
-    if (Kma) {
-      ma_bound <- subset2(prior, class = "ma", ls = px)$bound
+    if (get_ma(autocor)) {
       str_add(out$par) <- glue( 
         "  vector{ma_bound}[Kma{p}] ma{p};  // moving-average effects\n"
       )
@@ -179,97 +245,91 @@ stan_autocor <- function(bterms, prior) {
         prior, class = "ma", px = px, suffix = p
       )
     }
-    if (use_cov(autocor)) {
-      # if the user wants ARMA effects to be estimated using
-      # a covariance matrix for residuals
-      err_msg <- "Cannot use ARMA covariance matrices"
-      if (isTRUE(bterms$rescor)) {
-        stop2(err_msg, " when estimating 'rescor'.")
-      }
-      if (has_natural_residuals && length(names(bterms$dpars)) > 1L) {
-        stop2(err_msg, " when predicting distributional parameters.")
-      }
-      str_add(out$data) <- glue( 
-        "  // see the functions block for details\n",
-        "  int<lower=1> N_tg{p};\n",
-        "  int<lower=1> begin_tg{p}[N_tg{p}];\n", 
-        "  int<lower=1> end_tg{p}[N_tg{p}];\n", 
-        "  int<lower=1> nobs_tg{p}[N_tg{p}];\n"
+  }
+  if (is.cor_cosy(autocor)) {
+    # compound symmetry correlation structure
+    err_msg <- "Compound symmetry models are not implemented"
+    if (is.mixfamily(family)) {
+      stop2(err_msg, " for mixture models.") 
+    }
+    # most code is shared with ARMA covariance models
+    str_add(out$par) <- glue(
+      "  real<lower=0,upper=1> cosy{p};  // compound symmetry correlation\n"
+    )
+    str_add(out$prior) <- stan_prior(
+      prior, class = "cosy", px = px, suffix = p
+    )
+  }
+  if (use_cov(autocor)) {
+    # use correlation structures in covariance matrix parameterization
+    # optional for ARMA models and obligatory for compound symmetry models
+    err_msg <- "Cannot model residual covariance matrices via 'autocor'"
+    if (isTRUE(bterms$rescor)) {
+      stop2(err_msg, " when estimating 'rescor'.")
+    }
+    pred_other_pars <- any(!names(bterms$dpars) %in% c("mu", "sigma"))
+    if (has_natural_residuals && pred_other_pars) {
+      stop2(err_msg, " when predicting parameters other than 'mu' and 'sigma'.")
+    }
+    str_add(out$data) <- glue( 
+      "  // see the functions block for details\n",
+      "  int<lower=1> N_tg{p};\n",
+      "  int<lower=1> begin_tg{p}[N_tg{p}];\n", 
+      "  int<lower=1> end_tg{p}[N_tg{p}];\n", 
+      "  int<lower=1> nobs_tg{p}[N_tg{p}];\n"
+    )
+    if (!is.formula(bterms$adforms$se)) {
+      str_add(out$tdata_def) <- glue(
+        "  // no known standard errors specified by the user\n",
+        "  vector[N{p}] se2{p} = rep_vector(0, N{p});\n"
       )
-      if (!is.formula(bterms$adforms$se)) {
-        str_add(out$tdataD) <- glue(
-          "  vector[N{p}] se2{p} = rep_vector(0, N{p});\n"
-        )
-      }
-      str_add(out$tparD) <- glue(
-        "  matrix[max(nobs_tg{p}), max(nobs_tg{p})] chol_cov;\n"               
-      )
-      if (Kar && !Kma) {
-        cov_fun <- "ar1"
-        cov_args <- glue("ar{p}[1]")
-      } else if (!Kar && Kma) {
-        cov_fun <- "ma1"
-        cov_args <- glue("ma{p}[1]")
+    }
+    str_add(out$tpar_def) <- glue(
+      "  // cholesky factor of the autocorrelation matrix\n",
+      "  matrix[max(nobs_tg{p}), max(nobs_tg{p})] chol_cor;\n"               
+    )
+    if (is.cor_arma(autocor)) {
+      if (has_ar_only(autocor)) {
+        cor_fun <- "ar1"
+        cor_args <- glue("ar{p}[1]")
+      } else if (has_ma_only(autocor)) {
+        cor_fun <- "ma1"
+        cor_args <- glue("ma{p}[1]")
       } else {
-        cov_fun <- "arma1"
-        cov_args <- glue("ar{p}[1], ma{p}[1]")
-      }
-      sdpar <- str_if(has_natural_residuals, "sigma", "sderr")
-      str_add(out$tparC1) <- glue(
-        "  // compute residual covariance matrix\n",
-        "  chol_cov{p} = cholesky_cov_{cov_fun}", 
-        "({cov_args}, {sdpar}{p}, max(nobs_tg{p}));\n"
-      )
-      if (has_latent_residuals) {
-        str_add(out$par) <- glue(
-          "  vector[N{p}] zerr{p};  // unscaled residuals\n",
-          "  real<lower=0> sderr;  // SD of residuals\n"
-        )
-        str_add(out$tparD) <- glue(
-          "  vector[N{p}] err{p};  // actual residuals\n"
-        )
-        str_add(out$tparC1) <- glue(
-          "  // compute correlated residuals\n",
-          "  err{p} = scale_cov_err(",
-          "zerr{p}, chol_cov{p}, nobs_tg{p}, begin_tg{p}, end_tg{p});\n"
-        )
-        str_add(out$prior) <- glue(
-          "  target += normal_lpdf(zerr | 0, 1);\n"
-        )
-        str_add(out$prior) <- stan_prior(
-          prior, class = "sderr", px = px, suffix = p
-        )
-      }
-    } else {
-      err_msg <- "Please set cov = TRUE in ARMA correlation structures"
-      if (!has_natural_residuals) {
-        stop2(err_msg, " for family '", family$family, "'.")
+        cor_fun <- "arma1"
+        cor_args <- glue("ar{p}[1], ma{p}[1]")
       }
+    }
+    if (is.cor_cosy(autocor)) {
+      cor_fun <- "cosy"
+      cor_args <- glue("cosy{p}")
+    }
+    str_add(out$tpar_comp) <- glue(
+      "  // compute residual covariance matrix\n",
+      "  chol_cor{p} = cholesky_cor_{cor_fun}({cor_args}, max(nobs_tg{p}));\n"
+    )
+    if (has_latent_residuals) {
+      err_msg <- "Latent residuals are not implemented"
       if (is.btnl(bterms$dpars[["mu"]])) {
-        stop2(err_msg, " in non-linear models.")
+        stop2(err_msg, " for non-linear models.")
       }
-      if (!identical(family$link, "identity")) {
-        stop2(err_msg, " when using non-identity links.")
-      }
-      if (is.formula(bterms$adforms$se)) {
-        stop2(err_msg, " when including known standard errors.")
-      }
-      str_add(out$data) <- glue( 
-        "  int<lower=0> J_lag{p}[N{p}];\n"                
+      str_add(out$par) <- glue(
+        "  vector[N{p}] zerr{p};  // unscaled residuals\n",
+        "  real<lower=0> sderr;  // SD of residuals\n"
       )
-      str_add(out$modelD) <- glue(
-        "  // objects storing residuals\n",
-        "  matrix[N{p}, max_lag{p}] Err{p}",
-        " = rep_matrix(0, N{p}, max_lag{p});\n",
-        "  vector[N{p}] err{p};\n"
+      str_add(out$tpar_def) <- glue(
+        "  vector[N{p}] err{p};  // actual residuals\n"
       )
-      Y <- str_if(is.formula(bterms$adforms$mi), "Yl", "Y")
-      str_add(out$modelC2) <- glue(
-        "    // computation of ARMA correlations\n",
-        "    err{p}[n] = {Y}{p}[n] - mu{p}[n];\n",
-        "    for (i in 1:J_lag{p}[n]) {{\n",
-        "      Err{p}[n + 1, i] = err{p}[n + 1 - i];\n",
-        "    }}\n"
+      str_add(out$tpar_comp) <- glue(
+        "  // compute correlated residuals\n",
+        "  err{p} = scale_cov_err(",
+        "zerr{p}, sderr{p}, chol_cor{p}, nobs_tg{p}, begin_tg{p}, end_tg{p});\n"
+      )
+      str_add(out$prior) <- glue(
+        "  target += normal_lpdf(zerr | 0, 1);\n"
+      )
+      str_add(out$prior) <- stan_prior(
+        prior, class = "sderr", px = px, suffix = p
       )
     }
   }
@@ -288,18 +348,26 @@ stan_autocor <- function(bterms, prior) {
       stop2(err_msg, " when predicting 'sigma' or 'nu'.")
     }
     str_add(out$data) <- glue(
-      "  matrix[N{p}, N{p}] W{p};  // spatial weight matrix\n"                  
+      "  matrix[N{p}, N{p}] W{p};  // spatial weight matrix\n",
+      "  vector[N{p}] eigenW{p};  // eigenvalues of W{p}\n"
+    )
+    str_add(out$tdata_def) <- glue(
+      "  // the eigenvalues define the boundaries of the SAR correlation\n",
+      "  real min_eigenW{p} = min(eigenW{p});\n",
+      "  real max_eigenW{p} = max(eigenW{p});\n"
     )
     if (identical(autocor$type, "lag")) {
       str_add(out$par) <- glue( 
-        "  real<lower=0,upper=1> lagsar{p};  // SAR parameter\n"
+        "  // lag-SAR correlation parameter\n",
+        "  real<lower=min_eigenW{p},upper=max_eigenW{p}> lagsar{p};\n"
       )
       str_add(out$prior) <- stan_prior(
         prior, class = "lagsar", px = px, suffix = p
       )
     } else if (identical(autocor$type, "error")) {
       str_add(out$par) <- glue( 
-        "  real<lower=0,upper=1> errorsar{p};  // SAR parameter\n"
+        "  // error-SAR correlation parameter\n",
+        "  real<lower=min_eigenW{p},upper=max_eigenW{p}> errorsar{p};\n"
       )
       str_add(out$prior) <- stan_prior(
         prior, class = "errorsar", px = px, suffix = p
@@ -311,8 +379,8 @@ stan_autocor <- function(bterms, prior) {
     if (is.mixfamily(family)) {
       stop2(err_msg, " for mixture models.") 
     }
-    if (length(bterms$dpars[["mu"]]$nlpars)) {
-      stop2(err_msg, " in non-linear models.")
+    if (is.btnl(bterms$dpars[["mu"]])) {
+      stop2(err_msg, " for non-linear models.")
     }
     str_add(out$data) <- glue(
       "  // data for the CAR structure\n",
@@ -322,18 +390,17 @@ stan_autocor <- function(bterms, prior) {
       "  int<lower=1> edges1{p}[Nedges{p}];\n",
       "  int<lower=1> edges2{p}[Nedges{p}];\n"
     )
+    str_add(out$par) <- glue(
+      "  real<lower=0> sdcar{p};  // SD of the CAR structure\n"
+    )
+    str_add(out$prior) <- stan_prior(
+      prior, class = "sdcar", px = px, suffix = p
+    )
     if (autocor$type %in% c("escar", "esicar")) {
       str_add(out$data) <- glue(
         "  vector[Nloc{p}] Nneigh{p};\n",
         "  vector[Nloc{p}] eigenW{p};\n"
       )
-      str_add(out$par) <- glue(
-        "  // parameters for the CAR structure\n",
-        "  real<lower=0> sdcar{p};\n"
-      )
-      str_add(out$prior) <- stan_prior(
-        prior, class = "sdcar", px = px, suffix = p
-      )
     }
     if (autocor$type %in% "escar") {
       str_add(out$par) <- glue(
@@ -357,10 +424,10 @@ stan_autocor <- function(bterms, prior) {
       str_add(out$par) <- glue(
         "  vector[Nloc{p} - 1] zcar{p};\n"
       )
-      str_add(out$tparD) <- glue(
+      str_add(out$tpar_def) <- glue(
         "  vector[Nloc{p}] rcar{p};\n"                
       )
-      str_add(out$tparC1) <- glue(
+      str_add(out$tpar_comp) <- glue(
         "  // sum-to-zero constraint\n",
         "  rcar[1:(Nloc{p} - 1)] = zcar{p};\n",
         "  rcar[Nloc{p}] = - sum(zcar{p});\n"
@@ -380,13 +447,47 @@ stan_autocor <- function(bterms, prior) {
       # http://mc-stan.org/users/documentation/case-studies/icar_stan.html
       str_add(out$par) <- glue(
         "  // parameters for the ICAR structure\n",
-        "  vector[Nloc{p}] rcar{p};\n"
+        "  vector[Nloc{p}] zcar{p};\n"
+      )
+      str_add(out$tpar_def) <- glue(
+        "  // scaled parameters for the ICAR structure\n",
+        "  vector[Nloc{p}] rcar{p} = zcar{p} * sdcar{p};\n"
       )
       str_add(out$prior) <- glue(
-        "  target += -0.5 * dot_self(rcar{p}[edges1{p}]", 
-        " - rcar{p}[edges2{p}]);\n",
+        "  // improper prior on the spatial CAR component\n",
+        "  target += -0.5 * dot_self(zcar{p}[edges1{p}] - zcar{p}[edges2{p}]);\n",
         "  // soft sum-to-zero constraint\n",
-        "  target += normal_lpdf(sum(rcar{p}) | 0, 0.001 * Nloc{p});\n"
+        "  target += normal_lpdf(sum(zcar{p}) | 0, 0.001 * Nloc{p});\n"
+      )
+    } else if (autocor$type %in% "bym2") {
+      # BYM2 car based on the case study of Mitzi Morris
+      # http://mc-stan.org/users/documentation/case-studies/icar_stan.html
+      str_add(out$data) <- glue(
+        "  // scaling factor of the spatial CAR component\n",
+        "  real<lower=0> car_scale{p};\n"
+      )
+      str_add(out$par) <- glue(
+        "  // parameters for the BYM2 structure\n",
+        "  vector[Nloc{p}] zcar{p};  // spatial part\n",
+        "  vector[Nloc{p}] nszcar{p};  // non-spatial part\n",
+        "  // proportion of variance in the spatial part\n",
+        "  real<lower=0,upper=1> rhocar{p};\n"
+      )
+      str_add(out$tpar_def) <- glue(
+        "  // join the spatial and the non-spatial CAR component\n",
+        "  vector[Nloc{p}] rcar{p} = (sqrt(1 - rhocar{p}) * nszcar{p}", 
+        " + sqrt(rhocar{p} * inv(car_scale{p})) * zcar{p}) * sdcar{p};\n"
+      )
+      str_add(out$prior) <- stan_prior(
+        prior, class = "rhocar", px = px, suffix = p
+      )
+      str_add(out$prior) <- glue(
+        "  // improper prior on the spatial BYM2 component\n",
+        "  target += -0.5 * dot_self(zcar{p}[edges1{p}] - zcar{p}[edges2{p}]);\n",
+        "  // soft sum-to-zero constraint\n",
+        "  target += normal_lpdf(sum(zcar{p}) | 0, 0.001 * Nloc{p});\n",
+        "  // proper prior on the non-spatial BYM2 component\n",
+        "  target += normal_lpdf(nszcar | 0, 1);\n"
       )
     }
   }
@@ -405,7 +506,7 @@ stan_autocor <- function(bterms, prior) {
       "  matrix[N{p}, N{p}] V{p};  // known residual covariance matrix\n"
     )
     if (family$family %in% "gaussian") {
-      str_add(out$tdataD) <- glue(
+      str_add(out$tdata_def) <- glue(
         "  matrix[N{p}, N{p}] LV{p} = cholesky_decompose(V{p});\n"
       )
     }
@@ -447,7 +548,7 @@ stan_global_defs <- function(bterms, prior, ranef, cov_ranef) {
   }
   const <- family_info(bterms, "const")
   if (length(const)) {
-    str_add(out$tdataD) <- cglue("  {const};\n")
+    str_add(out$tdata_def) <- cglue("  {const};\n")
   }
   is_ordinal <- ulapply(families, is_ordinal)
   if (any(is_ordinal)) {
@@ -477,10 +578,11 @@ stan_global_defs <- function(bterms, prior, ranef, cov_ranef) {
     str_add(out$fun) <- glue(
       "  #include 'fun_normal_cov.stan'\n",
       "  #include 'fun_student_t_cov.stan'\n",
-      "  #include 'fun_cholesky_cov_ar1.stan'\n",
-      "  #include 'fun_cholesky_cov_ma1.stan'\n",
-      "  #include 'fun_cholesky_cov_arma1.stan'\n",
-      "  #include 'fun_scale_cov_err.stan'\n"
+      "  #include 'fun_scale_cov_err.stan'\n",
+      "  #include 'fun_cholesky_cor_ar1.stan'\n",
+      "  #include 'fun_cholesky_cor_ma1.stan'\n",
+      "  #include 'fun_cholesky_cor_arma1.stan'\n",
+      "  #include 'fun_cholesky_cor_cosy.stan'\n"
     )
   }
   if (any(ulapply(autocors, is.cor_sar))) {
@@ -614,22 +716,22 @@ stan_Xme <- function(meef, prior) {
     if (nzchar(g)) {
       Nme <- glue("Nme_{i}")
       str_add(out$data) <- glue(
-        "  int<lower=0> Nme_{i};\n",
-        "  int<lower=1> Jme_{i}[N];\n"
+        "  int<lower=0> Nme_{i};  // number of latent values\n",
+        "  int<lower=1> Jme_{i}[N];  // group index per observation\n"
       )
     } else {
       Nme <- "N"
     }
     str_add(out$data) <- glue(
-      "  int<lower=1> Mme_{i};\n"
+      "  int<lower=1> Mme_{i};  // number of groups\n"
     )
     str_add(out$data) <- cglue(
-      "  vector[{Nme}] Xn_{K};\n",
-      "  vector<lower=0>[{Nme}] noise_{K};\n"
+      "  vector[{Nme}] Xn_{K};  // noisy values\n",
+      "  vector<lower=0>[{Nme}] noise_{K};  // measurement noise\n"
     )
     str_add(out$par) <- cglue(
-      "  vector[Mme_{i}] meanme_{i};\n",
-      "  vector<lower=0>[Mme_{i}] sdme_{i};\n"
+      "  vector[Mme_{i}] meanme_{i};  // latent means\n",
+      "  vector<lower=0>[Mme_{i}] sdme_{i};  // latent SDs\n"
     )
     str_add(out$prior) <- glue(
       stan_prior(prior, "meanme", coef = coefs[K], suffix = usc(i)),
@@ -640,35 +742,42 @@ stan_Xme <- function(meef, prior) {
     )
     if (meef$cor[K[1]] && length(K) > 1L) {
       str_add(out$data) <- glue(
-        "  int<lower=1> NCme_{i};\n"
+        "  int<lower=1> NCme_{i};  // number of latent correlations\n"
       )
       str_add(out$par) <- glue(
-        "  matrix[Mme_{i}, {Nme}] zme_{i};\n",
+        "  matrix[Mme_{i}, {Nme}] zme_{i};  // standardized latent values\n",
+        "  // cholesky factor of the latent correlation matrix\n",
         "  cholesky_factor_corr[Mme_{i}] Lme_{i};\n"
       )
-      str_add(out$tparD) <- glue(
+      str_add(out$tpar_def) <- glue(
+        "  // obtain the actual latent values\n",
         "  matrix[{Nme}, Mme_{i}] Xme{i}", 
         " = rep_matrix(meanme_{i}', {Nme}) ", 
         " + (diag_pre_multiply(sdme_{i}, Lme_{i}) * zme_{i})';\n"
       )
-      str_add(out$tparD) <- cglue(
+      str_add(out$tpar_def) <- cglue(
+        "  // using separate vectors increases efficiency\n",
         "  vector[{Nme}] Xme_{K} = Xme{i}[, {K}];\n"
       )
       str_add(out$prior) <- glue(
         "  target += normal_lpdf(to_vector(zme_{i}) | 0, 1);\n",
         stan_prior(prior, "Lme", group = g, suffix = usc(i))
       )
-      str_add(out$genD) <- cglue(
+      str_add(out$gen_def) <- cglue(
+        "  // obtain latent correlation matrix\n",
         "  corr_matrix[Mme_{i}] Corme_{i}", 
         " = multiply_lower_tri_self_transpose(Lme_{i});\n",
         "  vector<lower=-1,upper=1>[NCme_{i}] corme_{i};\n"
       )
-      str_add(out$genC) <- stan_cor_genC(glue("corme_{i}"), glue("Mme_{i}"))
+      str_add(out$gen_comp) <- stan_cor_gen_comp(
+        cor = glue("corme_{i}"), ncol = glue("Mme_{i}")
+      )
     } else {
       str_add(out$par) <- cglue(
-        "  vector[{Nme}] zme_{K};\n"
+        "  vector[{Nme}] zme_{K};  // standardized latent values\n"
       )
-      str_add(out$tparD) <- cglue(
+      str_add(out$tpar_def) <- cglue(
+        "  // obtain the actual latent values\n",
         "  vector[{Nme}] Xme_{K} = ",
         "meanme_{i}[{K}] + sdme_{i}[{K}] * zme_{K};\n"
       )
@@ -730,7 +839,7 @@ stan_vector <- function(...) {
 # prepare Stan code for correlations in the generated quantities block
 # @param cor name of the correlation vector
 # @param ncol number of columns of the correlation matrix
-stan_cor_genC <- function(cor, ncol) {
+stan_cor_gen_comp <- function(cor, ncol) {
   Cor <- paste0(toupper(substring(cor, 1, 1)), substring(cor, 2))
   glue(
     "  // extract upper diagonal of correlation matrix\n", 
@@ -745,24 +854,33 @@ stan_cor_genC <- function(cor, ncol) {
 # indicates if a family-link combination has a built in 
 # function in Stan (such as binomial_logit)
 # @param family a list with elements 'family' and 'link'
-stan_has_built_in_fun <- function(family) {
+# @param cens_or_trunc is the model censored or truncated?
+stan_has_built_in_fun <- function(family, cens_or_trunc = FALSE) {
   stopifnot(all(c("family", "link") %in% names(family)))
   link <- family$link
   dpar <- family$dpar
   family <- family$family
-  log_families <- c(
-    "poisson", "negbinomial", "geometric", "com_poisson",
-    "zero_inflated_poisson", "zero_inflated_negbinomial",
-    "hurdle_poisson", "hurdle_negbinomial"
-  )
-  logit_families <- c(
-    "binomial", "bernoulli", "cumulative", "categorical",
-    "zero_inflated_binomial"
-  )
+  if (cens_or_trunc) {
+    # only few families have special lcdf and lccdf functions
+    log_families <- c("cox")
+    logit_families <- character(0)
+    logit_dpars <- character(0)
+  } else {
+    log_families <- c(
+      "poisson", "negbinomial", "geometric", "com_poisson",
+      "zero_inflated_poisson", "zero_inflated_negbinomial",
+      "hurdle_poisson", "hurdle_negbinomial", "cox"
+    )
+    logit_families <- c(
+      "binomial", "bernoulli", "cumulative", "categorical",
+      "zero_inflated_binomial"
+    )
+    logit_dpars <- c("zi", "hu")
+  }
   isTRUE(
     family %in% log_families && link == "log" ||
     family %in% logit_families && link == "logit" ||
-    isTRUE(dpar %in% c("zi", "hu")) && link == "logit"
+    isTRUE(dpar %in% logit_dpars) && link == "logit"
   )
 }
 
@@ -772,6 +890,11 @@ stan_all_vars <- function(x) {
   all_vars(x)
 }
 
+# transform names to be used as variable names in Stan
+make_stan_names <- function(x) {
+  gsub("\\.|_", "", make.names(x, unique = TRUE))
+}
+
 # checks if a model needs the kronecker product
 # @param ranef output of tidy_ranef
 # @param names_cov_ranef: names 'cov_ranef'
