@@ -1,36 +1,37 @@
 context("Tests for plotting functions")
 
 test_that("plotting functions don't throw unexpected errors", {
-  theme_set(theme_black())
-  fit <- brms:::rename_pars(brms:::brmsfit_example1)
+  ggplot2::theme_set(theme_black())
+  fit1 <- brms:::rename_pars(brms:::brmsfit_example1)
+  fit3 <- brms:::rename_pars(brms:::brmsfit_example3)
   
   # plot.brmsfit
-  expect_silent(p <- plot(fit, plot = FALSE))
-  expect_silent(p <- plot(fit, pars = "^b", plot = FALSE))
-  expect_silent(p <- plot(fit, pars = "^sd", plot = FALSE))
-  expect_error(plot(fit, pars = "123"),  "No valid parameters selected")
+  expect_silent(p <- plot(fit1, plot = FALSE))
+  expect_silent(p <- plot(fit1, pars = "^b", plot = FALSE))
+  expect_silent(p <- plot(fit1, pars = "^sd", plot = FALSE))
+  expect_error(plot(fit1, pars = "123"),  "No valid parameters selected")
   
   # stanplot.brmsfit
-  expect_silent(p <- stanplot(fit))
-  expect_silent(p <- stanplot(fit, pars = "^b"))
+  expect_silent(p <- stanplot(fit1))
+  expect_silent(p <- stanplot(fit1, pars = "^b"))
   expect_silent(p <- suppressMessages(
-    stanplot(fit, type = "trace", pars = "^b_")
+    stanplot(fit1, type = "trace", pars = "^b_")
   ))
-  expect_silent(p <- stanplot(fit, type = "hist", pars = "^sd_"))
-  expect_silent(p <- stanplot(fit, type = "dens"))
-  expect_silent(p <- stanplot(fit, type = "scatter",
-                              pars = parnames(fit)[2:3], 
+  expect_silent(p <- stanplot(fit1, type = "hist", pars = "^sd_"))
+  expect_silent(p <- stanplot(fit1, type = "dens"))
+  expect_silent(p <- stanplot(fit1, type = "scatter",
+                              pars = parnames(fit1)[2:3], 
                               exact_match = TRUE))
-  expect_silent(p <- stanplot(fit, type = "rhat", pars = "^b_"))
-  expect_silent(p <- stanplot(fit, type = "neff"))
-  expect_silent(p <- stanplot(fit, type = "acf"))
-  expect_silent(p <- stanplot(fit, type = "nuts_divergence"))
-  expect_error(stanplot(fit, type = "density"), "Invalid plot type")
-  expect_error(stanplot(fit, type = "hex"), 
+  expect_silent(p <- stanplot(fit1, type = "rhat", pars = "^b_"))
+  expect_silent(p <- stanplot(fit1, type = "neff"))
+  expect_silent(p <- stanplot(fit1, type = "acf"))
+  expect_silent(p <- stanplot(fit1, type = "nuts_divergence"))
+  expect_error(stanplot(fit1, type = "density"), "Invalid plot type")
+  expect_error(stanplot(fit1, type = "hex"), 
                "Exactly 2 parameters must be selected")
   
   # pairs.brmsfit
-  expect_s3_class(SW(pairs(fit, pars = parnames(fit)[1:3])), "bayesplot_grid")
+  expect_s3_class(SW(pairs(fit1, pars = parnames(fit1)[1:3])), "bayesplot_grid")
   
   # marginal_effects: manual checks of plotting method
   N <- 90
